package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.dao.ProfessorResultMapper;
import com.dayou.entity.ProfessorResult;
import com.dayou.entity.Question;
import com.dayou.entity.QuestionAnalysis;
import com.dayou.service.IProfessorResultService;
import com.dayou.service.IQuestionAnalysisService;
import com.dayou.service.websocket.WebSocketServer;
import com.dayou.vo.DocumentCycleVO;
import com.dayou.vo.ProfessorResultVO;
import com.dayou.vo.QuestionVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/13
 * created with IntelliJ IDEA.
 */
@Service
public class ProfessorResultServiceImpl extends ServiceImpl<ProfessorResultMapper, ProfessorResult> implements IProfessorResultService {

    @Autowired
    private ProfessorResultMapper professorResultMapper;

    @Autowired
    private IQuestionAnalysisService questionAnalysisService;

    @Autowired
    private WebSocketServer webSocketServer;

    @Transactional
    @Override
    public synchronized Boolean commitDocument(DocumentCycleVO documentCycleVO) {
        List<Question> questions = documentCycleVO.getResult();
        String professorNo = documentCycleVO.getProfessorNo();
        Long cycleId = documentCycleVO.getId();

        List<ProfessorResult> list = this.list(new LambdaQueryWrapper<ProfessorResult>()
                .eq(ProfessorResult::getCycleId, cycleId).eq(ProfessorResult::getProfessorNo, professorNo).eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (CollectionUtils.isEmpty(list)){
            List<ProfessorResult> collect = questions.stream().map(x -> {
                ProfessorResult professorResult = new ProfessorResult();
                professorResult.setProfessorNo(professorNo);
                professorResult.setScore(x.getScore());
                professorResult.setCycleId(cycleId);
                professorResult.setQuestionId(x.getId());
                return professorResult;
            }).collect(Collectors.toList());
            this.saveBatch(collect);
        }
        else {
            for (Question questionVO : questions){
                for (ProfessorResult professorResult :list){
                    if (professorResult.getQuestionId().equals(questionVO.getId())){
                        professorResult.setScore(questionVO.getScore());
                    }
                }
            }
            this.updateBatchById(list);
        }

        List<Long> questionIds = questions.stream().map(Question::getId).collect(Collectors.toList());
        analysisScore(cycleId,questionIds);
        webSocketServer.sendSocket("data");
        return Boolean.TRUE;
    }

    @Override
    public List<ProfessorResultVO> getResultList() {
        List<ProfessorResultVO> list = professorResultMapper.getResultList();
        return list;
    }

    @Override
    public List<ProfessorResultVO> getResultByCycleId(Long id) {
        List<ProfessorResultVO> list = professorResultMapper.getResultByCycleId(id);
        return list;
    }

    @Override
    public boolean removeScore(Long id) {
        ProfessorResult pr = this.getById(id);
        this.removeById(id);
        Long cycleId = pr.getCycleId();
        Long questionId = pr.getQuestionId();
        List<QuestionAnalysis> avgScores = professorResultMapper.getAvgScore(cycleId,questionId);
        return questionAnalysisService.saveOrUpdateBatch(avgScores);
    }

    private Boolean analysisScore(Long cycleId,List<Long> questionIds){
        for (Long questionId : questionIds){
            List<QuestionAnalysis> avgScores = professorResultMapper.getAvgScore(cycleId,questionId);
            questionAnalysisService.saveOrUpdateBatch(avgScores);
        }
        return Boolean.TRUE;
    }
}
