package com.dayou.service.websocket;

import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/20
 * created with IntelliJ IDEA.
 */
@Component
@ServerEndpoint(value="/ws",configurator = WebSocketConfig.class)
public class WebSocketServer {
    private static AtomicInteger atomicInteger = new AtomicInteger(0);

    private static ConcurrentHashMap<String,WebSocketServer> webSocketMap = new ConcurrentHashMap<>();

    private Session session;

    private static final String MSG = "{\"data\":\"*\"}";

    /**
     * 连接建立成功调用的方法
     * @param session
     */
    @OnOpen
    public void onOpen(Session session){
        atomicInteger.incrementAndGet();
        String sessionId = session.getId();
        this.session = session;
        System.out.println("建立连接成功 onOpen 线程数量："+atomicInteger.get()+" sessionID："+ session.getId());
        System.out.println("当前socket连接数量:" + webSocketMap.size());
        webSocketMap.put(sessionId,this);
        sendSocket("userSize");
    }

    /**
     * 连接关闭调用的方法
     * @param session
     */
    @OnClose
    public void onClose(Session session){
        System.out.println("一个客户端断开连接:" + session.getId());
        atomicInteger.incrementAndGet();
        webSocketMap.remove(session.getId());
        sendSocket("userSize");

    }

    /**
     * 接收到客户端消息后调用的方法
     * @param msg 客户端发送的消息
     * @param session
     */
    @OnMessage
    public void onMessage(String msg, Session session) throws IOException {
        atomicInteger.incrementAndGet();
        System.out.println("接收到客户端发送的消息 onMessage："+ msg+" 线程数量："+atomicInteger.get()+" sessionID："+session.getId());
        session.getBasicRemote().sendText(msg);
    }

    /**
     * 出现错误时调用的方法
     * @param session
     * @param throwable
     */
    @OnError
    public void onError(Session session, Throwable throwable){
        atomicInteger.incrementAndGet();
        System.out.println("错误 onError 线程数量："+atomicInteger.get()+" sessionID："+session.getId());
        throwable.printStackTrace();
    }

    public void sendSocket(String key){
        System.out.println("sessions："+webSocketMap.size());
        if (webSocketMap.size()>0){
            for(Map.Entry<String, WebSocketServer> map : webSocketMap.entrySet()){
                try {
                    if (key.equals("data")){
                        map.getValue().session.getBasicRemote().sendText(MSG);
                    }
                    else {
                        String userSize = "{\"userSize\":"+webSocketMap.size()+"}";
                        map.getValue().session.getBasicRemote().sendText(userSize);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
